@@ -3,7 +3,6 @@ import os
 import time
 import traceback
 import argparse
-from .exceptions import ConfigurationError
 from .inout import CravatReader
 from .inout import CravatWriter
 from .inout import AllMappingsParser
@@ -15,12 +14,14 @@ from .constants import crg_def
 from .constants import all_mappings_col_name
 from .constants import mapping_parser_name
 from .exceptions import InvalidData
+from .exceptions import ConfigurationError
 import sqlite3
-
-
+import json
+import cravat.cravat_util as cu
+import cravat.admin_util as au
 
 class BaseAnnotator(object):
-    
+
     valid_levels = ['variant','gene']
     valid_input_formats = ['crv','crx','crg']
     id_col_defs = {'variant':crv_def[0],
@@ -29,9 +30,12 @@ class BaseAnnotator(object):
                              'crx':[x['name'] for x in crx_def],
                              'crg':[x['name'] for x in crg_def]}
     required_conf_keys = ['level', 'output_columns']
-        
-    def __init__(self, cmd_args):
+
+    def __init__(self, cmd_args, status_writer, live=False):
         try:
+            if live:
+                return
+            self.status_writer = status_writer
             self.logger = None
             main_fpath = cmd_args[0]
             main_basename = os.path.basename(main_fpath)
@@ -41,19 +45,14 @@ class BaseAnnotator(object):
                 self.annotator_name = main_basename
             self.annotator_dir = os.path.dirname(main_fpath)
             self.data_dir = os.path.join(self.annotator_dir, 'data')
-            
-            # Load command line opts
             self.primary_input_path = None
             self.secondary_paths = None
             self.output_dir = None
             self.output_basename = None
             self.plain_output = None
             self.job_conf_path = None
+            # Load command line opts
             self.parse_cmd_args(cmd_args)
-            # Make output dir if it doesn't exist
-            if not(os.path.exists(self.output_dir)):
-                os.makedirs(self.output_dir)
-
             self._setup_logger()
             config_loader = ConfigLoader(self.job_conf_path)
             self.conf = config_loader.get_module_conf(self.annotator_name)
@@ -65,17 +64,22 @@ class BaseAnnotator(object):
                 self.annotator_display_name = self.conf['title']
             else:
                 self.annotator_display_name = os.path.basename(self.annotator_dir).upper()
+            if 'version' in self.conf:
+                self.annotator_version = self.conf['version']
+            else:
+                self.annotator_version = ''
             self.dbconn = None
             self.cursor = None
         except Exception as e:
             self._log_exception(e)
 
     def _log_exception(self, e, halt=True):
-        if self.logger:
-            self.logger.exception(e)
         if halt:
             raise e
-    
+        else:
+            if self.logger:
+                self.logger.exception(e)
+
     def _verify_conf(self):
         try:
             for k in self.required_conf_keys:
@@ -105,7 +109,7 @@ class BaseAnnotator(object):
                 self.conf['input_columns'] = self.default_input_columns[self.conf['input_format']]
         except Exception as e:
             self._log_exception(e)
-    
+
     def _define_cmd_parser(self):
         try:
             parser = argparse.ArgumentParser()
@@ -130,48 +134,63 @@ class BaseAnnotator(object):
                                 action='store_true',
                                 dest='plainoutput',
                                 help='Skip column definition writing')
+            parser.add_argument('--confs',
+                dest='confs',
+                default='{}',
+                help='Configuration string')
             self.cmd_arg_parser = parser
         except Exception as e:
             self._log_exception(e)
-    
+
     # Parse the command line arguments
     def parse_cmd_args(self, cmd_args):
         try:
             self._define_cmd_parser()
             parsed_args = self.cmd_arg_parser.parse_args(cmd_args[1:])
-                
             self.primary_input_path = os.path.abspath(parsed_args.input_file)
-            
             self.secondary_paths = {}
             if parsed_args.secondary_inputs:
                 for secondary_def in parsed_args.secondary_inputs:
                     sec_name, sec_path = secondary_def.split('@')
                     self.secondary_paths[sec_name] = os.path.abspath(sec_path)
-            
             self.output_dir = os.path.dirname(self.primary_input_path)
             if parsed_args.output_dir:
                 self.output_dir = parsed_args.output_dir
-    
+
             self.plain_output = parsed_args.plainoutput
-            
             self.output_basename = os.path.basename(self.primary_input_path)
             if parsed_args.name:
                 self.output_basename = parsed_args.name
-            
+            if self.output_basename != '__dummy__':
+                self.update_status_json_flag = True
+            else:
+                self.update_status_json_flag = False
             if parsed_args.conf:
                 self.job_conf_path = parsed_args.conf
+            self.confs = None
+            if parsed_args.confs is not None:
+                confs = parsed_args.confs.lstrip('\'').rstrip('\'').replace("'", '"')
+                self.confs = json.loads(confs)
         except Exception as e:
             self._log_exception(e)
-    
-    
+
     # Runs the annotator.
     def run(self):
+        if self.update_status_json_flag:
+            self.status_writer.queue_status_update('status', 'Started {} ({})'.format(self.conf['title'], self.annotator_name))
         try:
             start_time = time.time()
             self.logger.info('started: %s'%time.asctime(time.localtime(start_time)))
+            print('        {}: started at {}'.format(self.annotator_name, time.asctime(time.localtime(start_time))))
             self.base_setup()
-            for lnum, input_data, secondary_data in self._get_input():
+            last_status_update_time = time.time()
+            for lnum, line, input_data, secondary_data in self._get_input():
                 try:
+                    if self.update_status_json_flag:
+                        cur_time = time.time()
+                        if lnum % 10000 == 0 or cur_time - last_status_update_time > 3:
+                            self.status_writer.queue_status_update('status', 'Running {} ({}): line {}'.format(self.conf['title'], self.annotator_name, lnum))
+                            last_status_update_time = cur_time
                     if secondary_data == {}:
                         output_dict = self.annotate(input_data)
                     else:
@@ -188,67 +207,69 @@ class BaseAnnotator(object):
                             output_dict[col_name] = ''
                     self.output_writer.write_data(output_dict)
                 except Exception as e:
-                    self._log_runtime_exception(lnum, input_data, e)
-            
+                    self._log_runtime_exception(lnum, line, input_data, e)
             # This does summarizing.
             self.postprocess()
-            
             self.base_cleanup()
             end_time = time.time()
             self.logger.info('finished: {0}'.format(time.asctime(time.localtime(end_time))))
+            print('        {}: finished at {}'.format(self.annotator_name, time.asctime(time.localtime(end_time))))
             run_time = end_time - start_time
-            self.logger.info('runtime: {0:0.3f}'.format(run_time))
+            self.logger.info('runtime: {0:0.3f}s'.format(run_time))
+            print('        {}: runtime {:0.3f}s'.format(self.annotator_name, run_time))
+            if self.update_status_json_flag:
+                version = self.conf.get('version', 'unknown')
+                self.status_writer.queue_status_update('status', 'Finished {} ({})'.format(self.conf['title'], self.annotator_name))
         except Exception as e:
             self._log_exception(e)
-    
-        #self.log_handler.close()
-        #if self.output_basename == '__dummy__':
-        #    os.remove(self.log_path)
-            
+        if hasattr(self, 'log_handler'):
+            self.log_handler.close()
+        '''
+        if self.output_basename == '__dummy__':
+            os.remove(self.log_path)
+        '''
+
     def postprocess (self):
         pass
-    
-    def get_gene_summary_data (self, cf):
+
+    async def get_gene_summary_data (self, cf):
+        print('            {}: getting gene summary data'.format(self.annotator_name))
+        t = time.time()
+        hugos = await cf.get_filtered_hugo_list()
         cols = [self.annotator_name + '__' + coldef['name'] \
-                for coldef in self.conf['output_columns']]
-        cols[0] = 'base__hugo'
-        gene_collection = {}
-        for d in cf.get_variant_iterator_filtered_uids_cols(cols):
-            hugo = d['hugo']
-            if hugo == None:
-                continue
-            if hugo not in gene_collection:
-                gene_collection[hugo] = {}
-                for col in cols[1:]:
-                    gene_collection[hugo][col.split('__')[1]] = []
-            self.build_gene_collection(hugo, d, gene_collection)
+            for coldef in self.conf['output_columns'] \
+            if coldef['name'] != 'uid']
         data = {}
-        for hugo in gene_collection:
-            out = self.summarize_by_gene(hugo, gene_collection)
-            if out == None:
-                continue
-            row = []
-            for col in cols[1:]:
-                if col in out:
-                    val = out[col]
-                else:
-                    val = None
-                row.append(val)
+        t = time.time()
+        rows = await cf.get_variant_data_for_cols(cols)
+        rows_by_hugo = {}
+        t = time.time()
+        for row in rows:
+            hugo = row[-1]
+            if hugo not in rows_by_hugo:
+                rows_by_hugo[hugo] = []
+            rows_by_hugo[hugo].append(row)
+        t = time.time()
+        for hugo in hugos:
+            rows = rows_by_hugo[hugo]
+            input_data = {}
+            for i in range(len(cols)):
+                input_data[cols[i].split('__')[1]] = [row[i] for row in rows]
+            out = self.summarize_by_gene(hugo, input_data)
             data[hugo] = out
+        print('            {}: finished getting gene summary data in {:0.3f}s'.format(self.annotator_name, time.time() - t))
         return data
 
-    def _log_runtime_exception (self, lnum, input_data, e):
+    def _log_runtime_exception (self, lnum, line, input_data, e):
         try:
-            error_classname = e.__class__.__name__
-            err_line = '\t'.join([str(input_data[self._id_col_name]),
-                                  error_classname,
-                                  str(e)])
-            #self.invalid_file.write(err_line + '\n')
-            self.logger.exception(e)
-            '''
-            if not(isinstance(e,InvalidData)):
-                self._log_exception(e, halt=False)
-            '''
+            err_str = traceback.format_exc().rstrip()
+            lines = err_str.split('\n')
+            last_line = lines[-1]
+            err_str_log = '\n'.join(lines[:-1]) + '\n' + ':'.join(last_line.split(':')[:2])
+            if err_str_log not in self.unique_excs:
+                self.unique_excs.append(err_str_log)
+                self.logger.error(err_str_log)
+            self.error_logger.error('\n[{:d}]{}\n({})\n#'.format(lnum, line.rstrip(), str(e)))
         except Exception as e:
             self._log_exception(e, halt=False)
 
@@ -263,7 +284,7 @@ class BaseAnnotator(object):
             self.setup()
         except Exception as e:
             self._log_exception(e)
-    
+
     def _setup_primary_input(self):
         try:
             self.primary_input_reader = CravatReader(self.primary_input_path)
@@ -293,7 +314,7 @@ class BaseAnnotator(object):
                                                                   data_type=data_type)
         except Exception as e:
             self._log_exception(e)
-    
+
     def _setup_secondary_inputs(self):
         try:
             self.secondary_readers = {}
@@ -303,23 +324,20 @@ class BaseAnnotator(object):
                 num_expected = 0
             num_provided = len(self.secondary_paths)
             if num_expected > num_provided:
-                raise Exception('Too few secondary inputs. %d expected, ' +\
-                    '%d provided'%(num_expected, num_provided))
+                raise Exception(f'Too few secondary inputs. {num_expected} expected, {num_provided} provided')
             elif num_expected < num_provided:
                 raise Exception('Too many secondary inputs. %d expected, %d provided'\
                         %(num_expected, num_provided))
             for sec_name, sec_input_path in self.secondary_paths.items():
-                key_col = self.conf['secondary_inputs'][sec_name]\
-                                                       ['match_columns']\
-                                                       ['secondary']
-                use_columns = self.conf['secondary_inputs'][sec_name]['use_columns']
+                key_col = self.conf['secondary_inputs'][sec_name].get('match_columns',{}).get('secondary','uid')
+                use_columns = self.conf['secondary_inputs'][sec_name].get('use_columns',[])
                 fetcher = SecondaryInputFetcher(sec_input_path,
                                                 key_col,
                                                 fetch_cols=use_columns)
                 self.secondary_readers[sec_name] = fetcher
         except Exception as e:
             self._log_exception(e)
-    
+
     # Open the output files (.var, .gen, .ncd) that are needed
     def _setup_outputs(self):
         try:
@@ -355,12 +373,11 @@ class BaseAnnotator(object):
                                                    self.annotator_name)
                 self.output_writer.write_meta_line('displayname',
                                                    self.annotator_display_name)
+                self.output_writer.write_meta_line('version',
+                                                   self.annotator_version)
             skip_aggregation = []
             for col_index, col_def in enumerate(self.conf['output_columns']):
-                self.output_writer.add_column(col_index,
-                                              col_def['title'],
-                                              col_def['name'],
-                                              col_def['type'])
+                self.output_writer.add_column(col_index, col_def)
                 if not(col_def.get('aggregate', True)):
                     skip_aggregation.append(col_def['name'])
             if not(self.plain_output):
@@ -369,7 +386,7 @@ class BaseAnnotator(object):
                                                    ','.join(skip_aggregation))
         except Exception as e:
                 self._log_exception(e)
-    
+
     def _open_db_connection (self):
         db_dirs = [self.data_dir,
                    os.path.join('/ext', 'resource', 'newarch')]
@@ -379,21 +396,21 @@ class BaseAnnotator(object):
             if os.path.exists(db_path):
                 self.dbconn = sqlite3.connect(db_path)
                 self.cursor = self.dbconn.cursor()
-    
+
     def close_db_connection (self):
         self.cursor.close()
         self.dbconn.close()
-    
+
     def remove_log (self):
         pass
-    
+
     def get_uid_col (self):
         return self.conf['output_columns'][0]['name']
-        
+
     # Placeholder, intended to be overridded in derived class
     def setup(self):
         pass
-    
+
     def base_cleanup(self):
         try:
             self.output_writer.close()
@@ -403,42 +420,46 @@ class BaseAnnotator(object):
             self.cleanup()
         except Exception as e:
             self._log_exception(e)
-    
+
     # Placeholder, intended to be overridden in derived class
     def cleanup(self):
         pass
-    
+
     def remove_log_file (self):
         self.logger.removeHandler(self.log_handler)
         self.log_handler.flush()
         self.log_handler.close()
         os.remove(self.log_path)
-        
+
     # Setup the logging utility
     def _setup_logger(self):
         try:
-            #self.logger = logging.getLogger(self.annotator_name)
             self.logger = logging.getLogger('cravat.' + self.annotator_name)
-            '''
-            self.logger.propagate = False
-            self.logger.setLevel('INFO')
-            self.log_path = os.path.join(self.output_dir, 
-                                    '.'.join([self.output_basename, 
-                                             self.annotator_name, 
-                                             'log']))
-            self.log_handler = logging.FileHandler(self.log_path, mode='w')
-            formatter = logging.Formatter(
-                '%(name)20s%(lineno)6d   %(asctime)20s   %(message)s')
-            self.log_handler.setFormatter(formatter)
-            self.logger.addHandler(self.log_handler)
-            '''
+            if self.output_basename != '__dummy__':
+                self.log_path = os.path.join(self.output_dir, self.output_basename + '.log')
+                log_handler = logging.FileHandler(self.log_path, 'a')
+            else:
+                log_handler = logging.StreamHandler()
+            formatter = logging.Formatter('%(asctime)s %(name)-20s %(message)s', '%Y/%m/%d %H:%M:%S')
+            log_handler.setFormatter(formatter)
+            self.logger.addHandler(log_handler)
+            self.error_logger = logging.getLogger('error.' + self.annotator_name)
+            if self.output_basename != '__dummy__':
+                error_log_path = os.path.join(self.output_dir, self.output_basename + '.err')
+                error_log_handler = logging.FileHandler(error_log_path, 'a')
+            else:
+                error_log_handler = logging.StreamHandler()
+            formatter = logging.Formatter('SOURCE:%(name)-20s %(message)s')
+            error_log_handler.setFormatter(formatter)
+            self.error_logger.addHandler(error_log_handler)
         except Exception as e:
             self._log_exception(e)
+        self.unique_excs = []
 
     # Gets the input dict from both the input file, and 
     # any depended annotators depended annotator feature not complete.
     def _get_input(self):
-        for lnum, reader_data in self.primary_input_reader.loop_data('dict'):
+        for lnum, line, reader_data in self.primary_input_reader.loop_data():
             try:
                 input_data = {}
                 for col_name in self.conf['input_columns']:
@@ -448,22 +469,38 @@ class BaseAnnotator(object):
                         AllMappingsParser(input_data[all_mappings_col_name])
                 secondary_data = {}
                 for annotator_name, fetcher in self.secondary_readers.items():
-                    input_key_col = self.conf['secondary_inputs']\
-                                              [annotator_name]\
-                                               ['match_columns']\
-                                                ['primary']
+                    input_key_col = self.conf['secondary_inputs'][annotator_name].get('match_columns',{}).get('primary','uid')
                     input_key_data = input_data[input_key_col]
                     secondary_data[annotator_name] = fetcher.get(input_key_data)
-                yield lnum, input_data, secondary_data
+                yield lnum, line, input_data, secondary_data
             except Exception as e:
                 self._log_runtime_error(lnum, e)
                 continue
-    
+
     def annotate (self, input_data):
         sys.stdout.write('        annotate method should be implemented. ' +\
                 'Exiting ' + self.annotator_display_name + '...\n')
         exit(-1)
-        
+
+    def live_report_substitute (self, d):
+        import re
+        if 'report_substitution' not in self.conf:
+            return d
+        rs_dic = self.conf['report_substitution']
+        rs_dic_keys = list(rs_dic.keys())
+        for colname in d.keys():
+            if colname in rs_dic_keys:
+                value = d[colname]
+                if colname in ['all_mappings', 'all_so']:
+                    for target in list(rs_dic[colname].keys()):
+                        value = re.sub('\\b' + target + '\\b', rs_dic[colname][target], value)
+                else:
+                    if value in rs_dic[colname]:
+                        value = rs_dic[colname][value]
+                d[colname] = value
+        return d
+
+
 class SecondaryInputFetcher():
     def __init__(self,
                  input_path,
@@ -488,22 +525,22 @@ class SecondaryInputFetcher():
             self.fetch_cols = valid_cols
         self.data = {}
         self.load_input()
-    
+
     def load_input(self):
-        for _, all_col_data in self.input_reader.loop_data('dict'):
+        for _, line, all_col_data in self.input_reader.loop_data():
             key_data = all_col_data[self.key_col]
             if key_data not in self.data: self.data[key_data] = []
             fetch_col_data = {}
             for col in self.fetch_cols:
                 fetch_col_data[col] = all_col_data[col]
             self.data[key_data].append(fetch_col_data)
-    
+
     def get(self, key_data):
         if key_data in self.data:
             return self.data[key_data]
         else:
             return []
-        
+
     def get_values (self, key_data, key_column):
         ret = [v[key_column] for v in self.data[key_data]]
         return ret
