@@ -60,6 +60,8 @@ class BaseAnnotator(object):
                 self.annotator_display_name = self.conf['title']
             else:
                 self.annotator_display_name = os.path.basename(self.annotator_dir).upper()
+            if 'version' in self.conf:
+                self.annotator_version = self.conf['version']
             self.logger.info('Initialized %s' %self.annotator_name)
             
             self.dbconn = None
@@ -230,7 +232,7 @@ class BaseAnnotator(object):
         try:
             error_classname = e.__class__.__name__
             err_line = '\t'.join([input_data[self._id_col_name], error_classname, str(e)])
-            self.invalid_file.write(err_line + '\n')
+            #self.invalid_file.write(err_line + '\n')
             if not(isinstance(e,InvalidData)):
                 self._log_exception(e, halt=False)
         except Exception as e:
@@ -261,6 +263,7 @@ class BaseAnnotator(object):
                     raise ConfigurationError(err_msg)
                 else:
                     default_columns = self.default_input_columns[self.conf['input_format']]
+                    print('default_columns=', default_columns)
                     for col_name in requested_input_columns:
                         try:
                             col_index = default_columns.index(col_name)
@@ -341,6 +344,8 @@ class BaseAnnotator(object):
                                                    self.annotator_name)
                 self.output_writer.write_meta_line('displayname',
                                                    self.annotator_display_name)
+                self.output_writer.write_meta_line('version',
+                                                   self.annotator_version)
 #                 self.output_writer.write_names(self.annotator_name,
 #                                                self.annotator_display_name)
             skip_aggregation = []
@@ -355,7 +360,7 @@ class BaseAnnotator(object):
                 self.output_writer.write_definition()
                 self.output_writer.write_meta_line('no_aggregate',
                                                    ','.join(skip_aggregation))
-            self.invalid_file = open(self.invalid_path, 'w')
+            #self.invalid_file = open(self.invalid_path, 'w')
         except Exception as e:
                 self._log_exception(e)
     
@@ -383,7 +388,7 @@ class BaseAnnotator(object):
     def base_cleanup(self):
         try:
             self.output_writer.close()
-            self.invalid_file.close()
+            #self.invalid_file.close()
             if self.dbconn != None:
                 self.close_db_connection()
             self.cleanup()
@@ -415,7 +420,7 @@ class BaseAnnotator(object):
     # Gets the input dict from both the input file, and 
     # any depended annotators depended annotator feature not complete.
     def _get_input(self):
-        for lnum, reader_data in self.primary_input_reader.loop_data('dict'):
+        for lnum, line, reader_data in self.primary_input_reader.loop_data():
             try:
                 input_data = {}
                 for col_name in self.conf['input_columns']:
@@ -467,7 +472,7 @@ class SecondaryInputFetcher():
         self.load_input()
     
     def load_input(self):
-        for _, all_col_data in self.input_reader.loop_data('dict'):
+        for _, line, all_col_data in self.input_reader.loop_data():
             key_data = all_col_data[self.key_col]
             if key_data not in self.data: self.data[key_data] = []
             fetch_col_data = {}
