@@ -1,7 +1,9 @@
 import os
-import yaml
+import oyaml as yaml
 import copy
 from cravat import admin_util as au
+import shutil
+from cravat import constants
 
 class ConfigLoader():
     
@@ -18,21 +20,27 @@ class ConfigLoader():
         
     def _load_main_conf(self, build_all=True):
         self._main = {}
-        if os.path.exists(self.main_conf_path):
-            self._main = au.load_yml_conf(self.main_conf_path)
+        if os.path.exists(self.main_conf_path) == False:
+            shutil.copy(os.path.join(constants.packagedir, 'cravat.yml'), self.main_conf_path)
+        self._main = au.load_yml_conf(self.main_conf_path)
         if build_all:
             self._build_all()
         
     def _load_job_conf(self, build_all=True):
         self._job = {}
         if self.job_conf_path:
-            self._job = au.load_yml_conf(self.job_conf_path)
+            if os.path.exists(self.job_conf_path):
+                self._job = au.load_yml_conf(self.job_conf_path)
+            else:
+                print('Job conf file', self.job_conf_path, 'does not exist.')
+                exit()
         if build_all:
             self._build_all()
     
     def _load_module_conf(self, module_name, build_all=True):
         module_info = au.get_local_module_info(module_name)
-        self._modules[module_name] = au.load_yml_conf(module_info.conf_path)
+        if module_info is not None:
+            self._modules[module_name] = au.load_yml_conf(module_info.conf_path)
         if build_all:
             self._build_all()
             
@@ -43,6 +51,8 @@ class ConfigLoader():
         if self._main:
             self._all['cravat'] = copy.deepcopy(self._main)
         self._all = au.recursive_update(self._all, self._job)
+        if 'run' not in self._all:
+            self._all['run'] = {}
         
     def save(self, path, modules=[]):
         """
@@ -81,10 +91,19 @@ class ConfigLoader():
     def get_module_conf(self, module_name):
         if module_name not in self._modules:
             self._load_module_conf(module_name)
-        return self._all['modules'][module_name]
+        if 'modules' in self._all and module_name in self._all['modules']:
+            return self._all['modules'][module_name]
+        else:
+            return None
     
     def get_cravat_conf(self):
-        return self._all['cravat']
+        if 'cravat' not in self._all:
+            self._load_main_conf()
+        return self._main
+    
+    def get_modules_conf(self):
+        conf = self._all.get('modules', {})
+        return conf
     
     def get_cravat_conf_value (self, key):
         if 'cravat' in self._all:
@@ -94,10 +113,18 @@ class ConfigLoader():
                 return None
         else:
             return None
-        
-    def override_cravat_conf (self, cravat_conf):
-        self._all['cravat'] = au.recursive_update(self._all['cravat'],
-                                               cravat_conf)
+
+    def override_run_conf (self, run_conf):
+        self._all['run'] = au.recursive_update(self._all['run'], run_conf)
     
+    def override_cravat_conf (self, cravat_conf):
+        self._all['cravat'] = au.recursive_update(self._all['cravat'], cravat_conf)
+
+    def override_all_conf (self, conf):
+        self._all = au.recursive_update(self._all, conf)
+
     def get_local_module_confs (self):
         return self._all['modules']
+
+    def get_run_conf (self):
+        return self._all['run']
